function AppListAssistant(item)
{
	this.item = item;
	this.listModel = {items:[]};
	
	// spinner model
	this.spinnerModel = {spinning: true};
}

AppListAssistant.prototype.setup = function()
{
	// setup list title
	this.controller.get('listTitle').innerHTML = this.item.name;
	
	// setup spinner model
	this.controller.setupWidget('spinner', {spinnerSize: 'large'}, this.spinnerModel);
	
	// setup list widget
	this.controller.setupWidget('appList', { itemTemplate: "app-list/rowTemplate", swipeToDelete: false, reorderable: false }, this.listModel);
	Mojo.Event.listen(this.controller.get('appList'), Mojo.Event.listTap, this.listTapHandler.bindAsEventListener(this));
	
	// hide the list while we update ipkg
	this.controller.get('appList').style.display = "none";
	
	// load list items
	switch (this.item.list)
	{
		case 'list_upgrades':	IPKGService.list_upgrades(this.onList.bindAsEventListener(this));	break;
		case 'list':			IPKGService.list(this.onList.bindAsEventListener(this));			break;
		case 'list_installed':	IPKGService.list_installed(this.onList.bindAsEventListener(this));	break;
		case 'info':			IPKGService.info(this.onList.bindAsEventListener(this));			break;
		default: /* error */  break;
	}
}

AppListAssistant.prototype.listTapHandler = function(event)
{
	IPKGService.info(this.onInfo.bindAsEventListener(this), event.item.pkg);
}

AppListAssistant.prototype.onInfo = function(payload)
{
	if (!payload)
		console.log('error');
	else 
	{
		//console.log(payload.errorCode + ' - ' + payload.errorText);
		console.log(payload.info[0].Status);
		
		this.controller.showAlertDialog({
		    onChoose: function(value) {},
		    title: $L("Status"),
		    message: payload.info[0].Status,
		    choices:[{label:$L('Ok'), value:""}]
	    });
	}
}

AppListAssistant.prototype.onList = function(payload)
{
	if (!payload)
		// error / no apps
		return;
	else 
	{
		if (this.item.list == 'info')
		{
			for (var x = 0; x < payload.info.length; x++)
			{
				if (payload.info[x].Section == this.item.category) 
				{
					// push package to app list model
					this.listModel.items.push(
					{
						pkg:			payload.info[x].Package,
						version:		payload.info[x].Version,
						description:	payload.info[x].Description
					});
				}
			}
		}
		else
		{
			for (var pkg in payload) 
			{
				// push package to app list model
				this.listModel.items.push(
				{
					pkg:			pkg,
					version:		payload[pkg].version,
					description:	payload[pkg].description
				});
			}
		}
		
		// stop and hide the spinner
		this.spinnerModel.spinning = false;
		this.controller.modelChanged(this.spinnerModel);
		this.controller.get('spinner').style.display = "none";
		
		// show the list
		this.controller.get('appList').style.display = "inline";
		
		// update list widget
		this.listModel.items.sort(function(a, b)
		{
			if (a.description && b.description) 
			{
				return ((a.description.toLowerCase() < b.description.toLowerCase()) ? -1 : ((a.description.toLowerCase() > b.description.toLowerCase()) ? 1 : 0));
			}
			else
			{
				return -1;
			}
		});
		this.controller.get('appList').mojo.noticeUpdatedItems(0, this.listModel.items);
	}
}

AppListAssistant.prototype.activate = function(event) {}

AppListAssistant.prototype.deactivate = function(event) {}

AppListAssistant.prototype.cleanup = function(event) {}
